from argparse import ArgumentParser

from awslambdalocal import main

def parse_args():
    parser = ArgumentParser(
        description="Run AWS Lambda function  written in Python on local machine.",
        conflict_handler='resolve')
    parser.add_argument("file",
                        metavar="FILE", 
                        type=str, 
                        help="lambda function file name")
    parser.add_argument("-e", "--event", 
                        metavar="EVENT", 
                        type=str, 
                        help="event data file name")
    parser.add_argument("-h", "--handler",
                        metavar="HANDLER",
                        type=str,
                        default="handler",
                        help="lambda function handler name, default: \"handler\"")
    parser.add_argument("-t", "--timeout",
                        metavar="TIMEOUT",
                        type=int,
                        default=3,
                        help="seconds until lambda function timeout, default: 3")
    parser.add_argument('-p', '--profile',
                        type=str,
                        help='Read the AWS profile of the file',
                        default='default')
    parser.add_argument('-r', '--region',
                        type=str,
                        help='Sets the AWS region, defaults to us-east-1',
                        default='us-east-1')
    parser.add_argument('-w', '--watch',
                        type=int,
                        help='Starts lambda-local in watch mode listening to the specified port [1-65535].')
    return parser.parse_args()


if __name__ == "__main__":
    args = parse_args()
    main.run(args)


import uuid
import logging

from datetime import datetime, timedelta


class Context(object):

    def __init__(
        self, 
        timeout_in_seconds: int
    ) -> None:
        self.aws_request_id = uuid.uuid4()
        self.function_name = 'LambdaLocalMachine'
        self.function_version = "$LATEST"
        self.invoked_function_arn = "arn:aws:lambda:us-east-1:000000000000:function:LambdaLocalMachine"
        self.memory_limit_in_mb = "128"
        self.log_group_name = "/aws/lambda/LambdaLocalMachine"
        self.log_stream_name = f'{datetime.now().strftime("%Y/%m/%d")}/[$LATEST]{uuid.uuid4()}'
        self.identity = None
        self.client_context = None

        self._timeout_in_seconds = timeout_in_seconds
        self._duration = timedelta(seconds=timeout_in_seconds)
    
    def get_remaining_time_in_millis(self):
        if self._timelimit is None:
            raise Exception("Context not activated.")
        return millis_interval(datetime.now(), self._timelimit)

    def log(self, msg):
        print(msg)

    def _activate(self):
        self._timelimit = datetime.now() + self._duration
        return self


def millis_interval(start, end):
    """start and end are datetime instances"""
    diff = end - start
    millis = diff.days * 24 * 60 * 60 * 1000
    millis += diff.seconds * 1000
    millis += diff.microseconds / 1000
    return millis


class ContextFilter(logging.Filter):
    def __init__(self, context):
        super(ContextFilter, self).__init__()
        self.context = context

    def filter(self, record):
        record.aws_request_id = self.context.aws_request_id
        return True



import os
import sys
import logging

from dotenv import load_dotenv

from awslambdalocal.loader import FunctionLoader


logging.basicConfig(stream=sys.stdout,
                    level=logging.INFO,
                    format='[%(name)s - %(levelname)s - %(asctime)s] %(message)s')


def run(args):
    set_environment_variables(args.profile, args.region)

    if args.watch:
        start_lambda_watch(args)
    else:
        start_lambda(args)


def start_lambda(args):
    from awslambdalocal.event import read_event
    e = read_event(args.event)
    (result, err_type) = FunctionLoader(
        source=args.file,
        function_name=args.handler,
        timeout=args.timeout
    ).load(e)

    if err_type is not None:
        sys.exit(1)


def start_lambda_watch(args):
    from awslambdalocal import server

    loader = FunctionLoader(
        source=args.file,
        function_name=args.handler,
        timeout=args.timeout
    )
    server.start(loader, args.watch)


def set_environment_variables(profile, region):
    os.environ["AWS_PROFILE"] = profile
    os.environ["AWS_DEFAULT_REGION"] = region
    load_dotenv()



import json
import pickle

from http import HTTPStatus
from logging import Logger, getLogger
from http.server import BaseHTTPRequestHandler, HTTPServer

from awslambdalocal.loader import FunctionLoader


def start(
    loader: FunctionLoader,
    server_port: int = 8008,
    logger: Logger = getLogger()
):
    class RequestHandler(BaseHTTPRequestHandler):
        def do_POST(self):
            datalen = int(self.headers['Content-Length'])
            data = self.rfile.read(datalen)
            event = json.loads(data)
            (result, err_type) = loader.load(event)
            print(result)

            self.send_response(HTTPStatus.OK)
            self.send_header('Content-Type', 'application/json')
            self.end_headers()

            if result:
                self.wfile.write(pickle.dumps(result))
            elif err_type:
                self.wfile.write(pickle.dumps(err_type))

    server = HTTPServer(('localhost', server_port), RequestHandler)
    logger.info("Server started http://localhost:%s" % (server_port))

    try:
        server.serve_forever()
    except KeyboardInterrupt:
        pass

    server.server_close()
    logger.info("Server stopped.")
    


import os
import sys
import uuid
import json
import timeit
import logging
import traceback
import multiprocessing

from importlib import reload

from awslambdalocal.context import Context, ContextFilter
from awslambdalocal.timeout import TimeoutException, time_limit


class FunctionLoader:
    def __init__(
        self,
        source: str,
        timeout: int,
        function_name: str
    ) -> None:
        self.timeout = timeout

        file_path = os.path.abspath(source)
        file_directory = os.path.dirname(file_path)
        sys.path.append(file_directory)

        mod_name = 'request-' + str(uuid.uuid4)
        if sys.version_info.major == 2:
            import imp
            mod = imp.load_source(mod_name, source)
        elif sys.version_info.major == 3 and sys.version_info.minor >= 5:
            import importlib
            spec = importlib.util.spec_from_file_location(mod_name, source)
            mod = importlib.util.module_from_spec(spec)
            sys.modules[mod_name] = mod
            spec.loader.exec_module(mod)
        else:
            raise Exception("unsupported python version")

        self.__func = getattr(mod, function_name)

    def load(self, event: dict):
        context = Context(self.timeout)

        logger = logging.getLogger()
        logger.info("Event: {}".format(event))
        logger.info("START RequestId: {} Version: {}".format(
            context.aws_request_id, context.function_version))

        queue = multiprocessing.Queue()
        p = multiprocessing.Process(
            target=self.__execute_in_process,
            args=(queue, self.__func, event, context)
        )
        p.start()
        (result, err_type, duration) = queue.get()
        p.join()

        logger.info("END RequestId: {}".format(context.aws_request_id))
        duration = "{0:.2f} ms".format(duration)
        logger.info("REPORT RequestId: {}\tDuration: {}".format(
            context.aws_request_id, duration))
        if type(result) is TimeoutException:
            logger.error("RESULT:\n{}".format(result))
        else:
            logger.info("RESULT:\n{}".format(result))

        return (result, err_type)

    def __execute_in_process(self, queue, loader, event, context):
        start_time = timeit.default_timer()
        result, err_type = self.__execute(loader, event, context)
        end_time = timeit.default_timer()
        duration = (end_time - start_time) * 1000
        queue.put((result, err_type, duration))

    def __execute(self, func, event, context):
        err_type = None

        logger = logging.getLogger()
        log_filter = ContextFilter(context)
        logger.addFilter(log_filter)

        try:
            with time_limit(context._timeout_in_seconds):
                result = func(event, context._activate())
        except TimeoutException as err:
            result = err
            err_type = 1
        except:
            err = sys.exc_info()
            result = json.dumps({
                "errorMessage": str(err[1]),
                "stackTrace": traceback.format_tb(err[2]),
                "errorType": err[0].__name__
            }, indent=4, separators=(',', ': '))
            err_type = 0

        return result, err_type



import signal
import threading
from contextlib import contextmanager
import _thread


class TimeoutException(Exception):
    pass


@contextmanager
def time_limit(seconds):
    if hasattr(signal, "SIGALRM"):
        def signal_handler(signum, frame):
            raise TimeoutException("Timeout after {} seconds.".format(seconds))
        signal.signal(signal.SIGALRM, signal_handler)
        signal.alarm(seconds)
        try:
            yield
        finally:
            signal.alarm(0)
    else:
        timer = threading.Timer(seconds, lambda: _thread.interrupt_main())
        timer.start()
        try:
            yield
        except KeyboardInterrupt:
            raise TimeoutException("Timeout after {} seconds.".format(seconds))
        finally:
            timer.cancel()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['awslambdalocal']

package_data = \
{'': ['*']}

install_requires = \
['python-dotenv>=0.19.2,<0.20.0']

setup_kwargs = {
    'name': 'awslambdalocal',
    'version': '1.0.8',
    'description': 'A tool to simulate running an AWS Lambda locally',
    'long_description': '# awslambdalocal\nawslambdalocal is a tool to simulate running an AWS Lambda locally, for lambda functions in Python.\n\n\n## Table of Contents\n\n* [Requirements](#requirements)\n* [Installation](#install)\n* [About: CLI](#about-cli)\n    * [Positional Arguments](#positional-arguments)\n    * [Optional Arguments](#optional-arguments)\n    * [CLI Examples](#cli-examples)\n* [Tutorials](#tutorials)\n    * [Debug Python in VSCode](#debug-python-in-vscode)\n\n\n## Requirements\n\n* Python => 3.8\n* Poetry => 1.1.12 or another package manager that supports direct git dependencies\n\n\n## Install\n\nTo install awslambdalocal, we recommend adding it to your pyproject.toml in the dev-dependencies section as shown in the example below.\n\n```bash\npip install awslambdalocal\n```\n**Obs.:** We recommend using Poetry. See https://python-poetry.org/docs/ \n\n\n## About: CLI\n\n### Positional Arguments:\n| Argument    | Description                                                                  |\n|-------------|------------------------------------------------------------------------------|\n| file        | Specify Lambda function file name                                            |\n\n### Optional Arguments:\n| Argument    | Description                                                                  |\n|-------------|------------------------------------------------------------------------------|\n| --help      | Show this help message and exit                                              |\n| -e          | Specify Event data file name. REQUIRED without param -w                      |\n| -h          | Lambda function handler name. Default is "handler"                           |\n| -p          | Read the AWS profile of the file.                                            |\n| -r          | Sets the AWS region, defaults to us-east-1.                                  |\n| -t          | Sets lambda timeout. default: 3                                              |\n| -w          | Starts lambda-local in watch mode listening to the specified port [1-65535]. |\n\n\n### CLI Examples\n```sh\n# Simple usage\npyhton -m awslambdalocal main.py test-event.json\n\n# Input all arguments\npyhton -m awslambdalocal main.py test-event.json -p my_profile -r my_region -h lambda_handler -t 30\n```\n\n\n## Tutorials\n---\nThis session contains a collection of tutorials.\n\n### Debug Python in VSCode\nTo use vscode debug with awslambdalocal follow the steps below\n\n1. Click run and debug\n2. Click create a launch.json file\n\n    ![](https://github.com/miqueiasbrs/py-aws-lambda-local/raw/master/docs/step_1.png)\n3. Choose Python\n\n    ![](https://github.com/miqueiasbrs/py-aws-lambda-local/raw/master/docs/step_2.png)\n4. Choose Module\n\n    ![](https://github.com/miqueiasbrs/py-aws-lambda-local/raw/master/docs/step_3.png)\n5. Set the module name "awslambdalocal"\n\n    ![](https://github.com/miqueiasbrs/py-aws-lambda-local/raw/master/docs/step_4.png)\n6. After this process, VSCode will create a file called launch.json in the .vscode folder located at the root of the project\n\n    ![](https://github.com/miqueiasbrs/py-aws-lambda-local/raw/master/docs/step_5.png)\n6. Copy and paste the json below into the launch.json file, this file aims to call the awslambdalocal module and passes the necessary and optional parameters as arguments\n\n    ```json\n    {\n        // Use o IntelliSense para saber mais sobre os atributos possíveis.\n        // Focalizar para exibir as descrições dos atributos existentes.\n        // Para obter mais informações, acesse: https://go.microsoft.com/fwlink/?linkid=830387\n        "version": "0.2.0",\n        "configurations": [\n            {\n                "name": "Lambda Local", // Debug configuration name\n                "type": "python", // Type of configuration. Python, Node and etc.\n                "request": "launch",\n                "module": "awslambdalocal", // Module that will be called,\n                "cwd": "${workspaceFolder}", // Your project\'s root folder\n                "args": [\n                    "file_python.py", // Main file that will be called by lambda\n                    "your_test_event.json", //Input in json format that will be received by lambda\n                    // Optional args ...\n                    "-h",\n                    "handler",\n                    "-p",\n                    "your_profile",\n                    "-r",\n                    "us-east-1"\n                ]\n            }\n        ]\n    }\n    ```',
    'author': 'Miqueias BRS',
    'author_email': 'miqueias@capybaracode.com.br',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
